package edu.georgetown.gucs.experiment;

import java.util.List;

import edu.georgetown.gucs.utility.Global;

/**
 * Provides values for confidence interval
 * 
 * @author Clay Shields
 * 
 */
class Statistics {

  private static double ninety_five_array[] = { 4.3027, 3.1824, 2.7765, 2.5706, 2.4469, 2.3646, 2.306, 2.2622, 2.2281,
      2.201, 2.1788, 2.1604, 2.1448, 2.1315, 2.1199, 2.1098, 2.1009, 2.093, 2.086, 2.0796, 2.0739, 2.0687, 2.0639,
      2.0595, 2.0555, 2.0518, 2.0484, 2.0452, 2.0423, 2.0395, 2.0369, 2.0345, 2.0322, 2.0301, 2.0281, 2.0262, 2.0244,
      2.0227, 2.0211, 2.0195, 2.0181, 2.0167, 2.0154, 2.0141, 2.0129, 2.0117, 2.0106, 2.0096, 2.0086, 2.0076, 2.0066,
      2.0057, 2.0049, 2.004, 2.0032, 2.0025, 2.0017, 2.001, 2.0003, 1.9996, 1.999, 1.9983, 1.9977, 1.9971, 1.9966,
      1.996, 1.9955, 1.9949, 1.9944, 1.9939, 1.9935, 1.993, 1.9925, 1.9921, 1.9917, 1.9913, 1.9908, 1.9905, 1.9901,
      1.9897, 1.9893, 1.989, 1.9886, 1.9883, 1.9879, 1.9876, 1.9873, 1.987, 1.9867, 1.9864, 1.9861, 1.9858, 1.9855,
      1.9852, 1.985, 1.9847, 1.9845, 1.9842, 1.984 };

  /**
   * Provides a value for the confidence interval
   * 
   * @param degreeOfFreedom
   *          the <code>int</code> degree of freedom to use for the confidence interval
   * 
   * @return the <code>double</code> value for the confidence interval
   */
  static double getTValueNinetyFive(int degreeOfFreedom) {
    if (degreeOfFreedom < 2) {
      Global.monitor.printErrorMessage("Statistics",
          "The degree of freedom must be 2 or greater. Returning a made-up value for output despite too few trials.");
      Global.monitor.writeToBuffer("Statistics", "getTValueNinetyFive",
          "degreeOfFreedom < 2; returning 6.0 (made-up value) for output with too few trials.");
      return 6.0;
    }
    int index = degreeOfFreedom - 2;
    if (index > 98) {
      return 1.96;
    }
    return ninety_five_array[index];
  }

  /**
   * Provides the mean for the given values
   * 
   * @param values
   *          the list of double values
   * 
   * @return the <code>double</code> mean; returns Double.NaN if values is empty
   */
  static double getMean(List<Double> values) {
    if (values.size() > 0) {
      double sum = 0.0;
      for (Double element : values) {
        sum += element;
      }
      return sum / values.size();
    }
    return Double.NaN;
  }

  /**
   * Provides the standard deviation for the given values
   * 
   * @param values
   *          the list of double values
   * @param mean
   *          the <code>double</code> mean of the values
   * 
   * @return the <code>double</code> standard deviation; returns Double.NaN if values is empty and 0.0 is values has a
   *         size of 1
   */
  static double getStandardDeviation(List<Double> values, double mean) {
    if (values.size() > 0) {
      if (values.size() > 1) {
        double sum = 0.0;
        for (Double element : values) {
          sum += Math.pow((element - mean), 2);
        }
        return Math.sqrt((1 / (double) (values.size() - 1)) * sum);
      }
      return 0.0;
    }
    return Double.NaN;
  }
}
