package edu.georgetown.gucs.fingerprinter;

import java.io.File;
import java.util.Map;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.List;
import java.util.ArrayList;

import edu.georgetown.gucs.tokenizers.TokenizerList;
import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.fingerprinter.Fingerprinter;
import edu.georgetown.gucs.fingerprinter.BulkFingerprinter;
import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.FileLister;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.ObjectBuilder;
import edu.georgetown.gucs.utility.Pair;
import edu.georgetown.gucs.utility.Token;
import edu.georgetown.gucs.utility.Converter;


/**
 * A thread worker for doing n^2 matching over all files in a 
 * directory.
 * 
 * Based in  part on code from this answer:
 * http://stackoverflow.com/questions/8974638/blocking-queue-and-multi-threaded-consumer-how-to-know-when-to-stop
 *
 * @author Clay Shields
 * 
 */

public class BulkFingerprinterWorker extends Thread {

    private Fingerprinter fingerprinter;
    private List<Pair<byte[], Token>> results;
    private FileLister fileProducer;
    private File poisonPill;
    private Dictionary ourDictionary;
    private BulkFingerprinter parent;


    // Takes a file and a dictionary and refernce to a Map to add the fp to
    // Creates a fingerprint
    // Returns the file name and resulting fingerprint to the parent

    public BulkFingerprinterWorker(String fingerprinterName, Dictionary dictionary, BulkFingerprinter parent, FileLister fileProducer, File poison){
	this.fingerprinter = (Fingerprinter) ObjectBuilder.buildObject(fingerprinterName);
	this.fingerprinter.setDictionary(dictionary);
	this.parent = parent;
	this.fileProducer = fileProducer;
	this.poisonPill = poison;

	results = new ArrayList<Pair<byte[], Token>>();
    }

    @Override
	public void run(){ 	
	
	while(true) {

		File file = fileProducer.takeFile();
		if (file != poisonPill){
		    List<Pair<byte[],Token>> fp = fingerprinter.computeFingerprint(file.getPath());
		    for (Pair<byte[], Token> currentFP : fp){			
			//System.err.println("File: " + file + " FP: " + Converter.asHex(currentFP.getA()));
			results.add(currentFP);
		    }


		}
		else {
		    parent.putResults(results);
		    fileProducer.putPoison(poisonPill);
		    break;
		}
	}
	//System.err.println("Ending thread ");
	return;
    }


}