
package edu.georgetown.gucs.utility;

import java.io.File;
import java.util.concurrent.BlockingQueue;
import java.util.Arrays;
import java.util.ArrayList;

/**
 * Thread for figuring out when the FileListerWorker threads are done and 
 * need to be let go
 *
 * @author Clay Shields
 */

class FileListerHR extends Thread {

    private ArrayList<FileListerWorker> threads;
    private BlockingQueue<File> sharedDirectoryQueue;
    private FileLister parent;
    private File poison;

    /**
     * Constructor for the HR thread
     * 
     * @param threadArray
     *          an array list that holds the FileListerWorkers
     * @param sharedDirectoryQueue
     *         the shared queue that holds the list of directories to be processes
     */
    FileListerHR(ArrayList<FileListerWorker> theThreads, BlockingQueue<File> theQueue, 
		 FileLister theParent, File thePoison){
	this.threads = theThreads;
	this.sharedDirectoryQueue = theQueue;
	this.parent = theParent;
	this.poison = thePoison;
    }


    /**
     * Adds the file to the existing fileLister
     */
    @Override
	public void run() {
	
	boolean done = false;

	try {		
	    while (!done){
		// Sleep a bit while the threads run
		Thread.sleep(100);
		// Check if the queue is empty and all threads in waiting state
		if (sharedDirectoryQueue.peek() == null){
		    int waitingThreads = 0;
		    for (int i = 0; i < threads.size(); i++){
			if (threads.get(i).getState() == Thread.State.valueOf("WAITING")) {
			    waitingThreads++;
			}
		    }
		    // If so, then we are done loading.
		    if (waitingThreads == threads.size()){
			parent.setLoaded();
			sharedDirectoryQueue.put(poison);
			// The next line doens't matter, but it makes FindBugs happy
			done = true;
		 
			Global.monitor.writeToBuffer("FileListerHR", "run", "Detected end" );
						
			return;

		    }
		    else {
			Global.monitor.writeToBuffer("FileListerHR", "run", "Continue" );
		    }
		}
	    }	    
	}
	catch (InterruptedException e){
	    ExceptionHandler.handleExceptionExit(e);
	}
    }
    
}
